var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { jsx as _jsx } from "react/jsx-runtime";
import { useState, useEffect, useMemo, useRef } from 'react';
import isEqual from 'lodash.isequal';
import MultiSelectField from '../../../../core/MultiSelectField';
import SelectField from '../../../../core/SelectField';
import replaceTemplateWithValues from '../../../../helpers/replaceTemplateWithValues';
const renderOptionLabel = (option, strings) => {
    return ((option.name && strings[option.name]) || option.name || option.label || '');
};
export default function SelectionInput2({ label, value, token, multiple = false, searchable = false, selectable = false, options: defaultOptions = [], onInputStateChange, optionsUrl: optionsUrlTemplate, optionsUrlHttpHeaders: optionsUrlHttpHeadersTemplate, sortable, helperText, error, disabled, onChange, strings, parentValue, inputState, parentProperties, }) {
    const [options, setOptions] = useState(defaultOptions);
    const [fetchOptionsError, setFetchOptionsError] = useState(false);
    const optionsUrl = useMemo(() => {
        if (!optionsUrlTemplate) {
            return { result: '', missingDependencies: [] };
        }
        return replaceTemplateWithValues(optionsUrlTemplate, parentValue, parentProperties, token);
    }, [optionsUrlTemplate, parentValue, parentProperties, token]);
    const headers = useMemo(() => {
        if (!optionsUrlHttpHeadersTemplate) {
            return { result: {}, missingDependencies: [] };
        }
        const headersReplaced = {};
        const allMissingDependencies = [];
        for (const header of optionsUrlHttpHeadersTemplate) {
            const { result, missingDependencies } = replaceTemplateWithValues(header.value, parentValue, parentProperties, token);
            if (missingDependencies.length > 0) {
                allMissingDependencies.push(...missingDependencies);
            }
            else {
                headersReplaced[header.key] = result;
            }
        }
        return {
            result: headersReplaced,
            missingDependencies: allMissingDependencies,
        };
    }, [optionsUrlHttpHeadersTemplate, parentValue, parentProperties, token]);
    const checkDefaultOptions = useMemo(() => (options) => {
        const isValueUnset = value === null || value === undefined;
        const hasOptions = options.length > 0;
        if (!hasOptions)
            return;
        const isValueInOptions = multiple
            ? Array.isArray(value) &&
                value.every((v) => options.some((opt) => opt.value === v))
            : options.some((opt) => opt.value === value);
        const shouldSetValueToDefault = isValueUnset || !isValueInOptions;
        if (shouldSetValueToDefault) {
            const defaultValues = options
                .filter((opt) => opt.default)
                .map((opt) => opt.value);
            // Auto-select the first option if no default is set. If we don't it results
            // in an unexpected UI state where the first value appears selected but the value
            // is still undefined. Multi-selects don't have this problem though so we are only
            // doing this for single selects.
            if (defaultValues.length === 0 && options.length > 0 && !multiple) {
                defaultValues.push(options[0].value);
            }
            if (defaultValues.length > 0) {
                if (multiple) {
                    onChange(defaultValues);
                }
                else {
                    onChange(defaultValues[0]);
                }
            }
        }
    }, [value, multiple, onChange]);
    // Track previous options URL and headers to avoid fetching options multiple times.
    const prevOptionsUrl = useRef();
    const prevHeaders = useRef();
    // Fetch options if URL is provided.
    useEffect(() => {
        // If no options URL is provided, do nothing.
        if (!optionsUrl.result)
            return;
        // If there are missing dependencies for the options URL or headers, do nothing.
        if (optionsUrl.missingDependencies.length > 0)
            return;
        if (headers.missingDependencies.length > 0)
            return;
        // If the options URL or headers have changed, fetch the options.
        const didOptionsUrlChange = optionsUrl.result !== prevOptionsUrl.current;
        const didHeadersChange = !isEqual(headers, prevHeaders.current);
        if (!didOptionsUrlChange && !didHeadersChange)
            return;
        const fetchOptions = () => __awaiter(this, void 0, void 0, function* () {
            try {
                prevOptionsUrl.current = optionsUrl.result;
                prevHeaders.current = headers.result;
                const response = yield fetch(optionsUrl.result, {
                    headers: headers.result,
                });
                if (!response.ok) {
                    throw new Error(`Invalid status ${response.status}`);
                }
                const options = yield response.json();
                // Ensure options is an array of objects with a value and label.
                const isValidOptions = Array.isArray(options) &&
                    options.every((opt) => {
                        return (typeof opt === 'object' &&
                            typeof opt.value === 'string' &&
                            typeof opt.label === 'string');
                    });
                if (isValidOptions) {
                    // Set the default value before the options. This fixes an issue with
                    // multi-selects not ordering the selected items to the top of the list.
                    checkDefaultOptions(options);
                    setOptions(options);
                }
                else {
                    throw new Error(`Invalid options received from ${optionsUrl.result}: ${JSON.stringify(options)}`);
                }
            }
            catch (error) {
                const errorMessage = error instanceof Error ? error.message : String(error);
                console.error(`Failed to fetch options at ${optionsUrl.result}: ${errorMessage}`);
                setFetchOptionsError(true);
            }
        });
        fetchOptions();
    }, [optionsUrl, headers, checkDefaultOptions]);
    // Render multi-select.
    if (multiple) {
        // Ensure it's an array to appease TS.
        const multiSelectValue = Array.isArray(value)
            ? value
            : Array.from(value || []);
        const selectedOptionValue = inputState &&
            inputState.selectedOption &&
            inputState.selectedOption.value;
        return (_jsx(MultiSelectField, Object.assign({ label: label, value: multiSelectValue, helperText: helperText, error: error || fetchOptionsError, disabled: disabled, searchable: searchable, selectable: selectable, onChange: onChange, sortable: sortable }, { children: options.map((opt, index) => (_jsx(MultiSelectField.Option, { value: opt.value, label: renderOptionLabel(opt, strings), rightLabel: opt.rightLabel, selected: selectable && selectedOptionValue === opt.value, onSelect: selectable
                    ? (option) => onInputStateChange({ selectedOption: option })
                    : undefined }, index))) }), void 0));
    }
    // Render single-select.
    const singleSelectValue = Array.isArray(value) ? value[0] : value;
    // Pick the first option if no value is selected.
    const selectValue = singleSelectValue || (options[0] ? options[0].value : '');
    return (_jsx(SelectField, Object.assign({ label: label, value: selectValue, onChange: onChange, helperText: helperText, error: error || fetchOptionsError, disabled: disabled, native: true }, { children: options.map((opt, index) => (_jsx("option", Object.assign({ value: opt.value }, { children: renderOptionLabel(opt, strings) }), index))) }), void 0));
}
//# sourceMappingURL=SelectionInput2.js.map